/*
    FreeRTOS V6.0.4 - Copyright (C) 2010 Real Time Engineers Ltd.

    ***************************************************************************
    *                                                                         *
    * If you are:                                                             *
    *                                                                         *
    *    + New to FreeRTOS,                                                   *
    *    + Wanting to learn FreeRTOS or multitasking in general quickly       *
    *    + Looking for basic training,                                        *
    *    + Wanting to improve your FreeRTOS skills and productivity           *
    *                                                                         *
    * then take a look at the FreeRTOS eBook                                  *
    *                                                                         *
    *        "Using the FreeRTOS Real Time Kernel - a Practical Guide"        *
    *                  http://www.FreeRTOS.org/Documentation                  *
    *                                                                         *
    * A pdf reference manual is also available.  Both are usually delivered   *
    * to your inbox within 20 minutes to two hours when purchased between 8am *
    * and 8pm GMT (although please allow up to 24 hours in case of            *
    * exceptional circumstances).  Thank you for your support!                *
    *                                                                         *
    ***************************************************************************

    This file is part of the FreeRTOS distribution.

    FreeRTOS is free software; you can redistribute it and/or modify it under
    the terms of the GNU General Public License (version 2) as published by the
    Free Software Foundation AND MODIFIED BY the FreeRTOS exception.
    ***NOTE*** The exception to the GPL is included to allow you to distribute
    a combined work that includes FreeRTOS without being obliged to provide the
    source code for proprietary components outside of the FreeRTOS kernel.
    FreeRTOS is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
    more details. You should have received a copy of the GNU General Public 
    License and the FreeRTOS license exception along with FreeRTOS; if not it 
    can be viewed here: http://www.freertos.org/a00114.html and also obtained 
    by writing to Richard Barry, contact details for whom are available on the
    FreeRTOS WEB site.

    1 tab == 4 spaces!

    http://www.FreeRTOS.org - Documentation, latest information, license and
    contact details.

    http://www.SafeRTOS.com - A version that is certified for use in safety
    critical systems.

    http://www.OpenRTOS.com - Commercial support, development, porting,
    licensing and training services.
*/

/*
 * Creates all the demo application tasks, then starts the scheduler.  The WEB
 * documentation provides more details of the standard demo application tasks.
 * In addition to the standard demo tasks, the following tasks and tests are
 * defined and/or created within this file:
 *
 * "Fast Interrupt Test" - A high frequency periodic interrupt is generated
 * using a free running timer to demonstrate the use of the 
 * configKERNEL_INTERRUPT_PRIORITY configuration constant.  The interrupt 
 * service routine measures the number of processor clocks that occur between
 * each interrupt - and in so doing measures the jitter in the interrupt 
 * timing.  The maximum measured jitter time is latched in the usMaxJitter 
 * variable, and displayed on the LCD by the 'Check' as described below.  
 * The fast interrupt is configured and handled in the timer_test.c source 
 * file.
 *
 * "LCD" task - the LCD task is a 'gatekeeper' task.  It is the only task that
 * is permitted to access the LCD directly.  Other tasks wishing to write a
 * message to the LCD send the message on a queue to the LCD task instead of 
 * accessing the LCD themselves.  The LCD task just blocks on the queue waiting 
 * for messages - waking and displaying the messages as they arrive.  The LCD
 * task is defined in lcd.c.  
 * 
 * "Check" task -  This only executes every three seconds but has the highest 
 * priority so is guaranteed to get processor time.  Its main function is to 
 * check that all the standard demo tasks are still operational.  Should any
 * unexpected behaviour within a demo task be discovered the 'check' task will
 * write "FAIL #n" to the LCD (via the LCD task).  If all the demo tasks are 
 * executing with their expected behaviour then the check task writes the max
 * jitter time to the LCD (again via the LCD task), as described above.
 */

/* Standard includes. */
#include <stdio.h>

/* Scheduler includes. */
#include "FreeRTOS.h"
#include "task.h"
#include "queue.h"
#include "semphr.h"
#include "croutine.h"

#include "..\include\defs.h"
#include "lcd.h"
#include "io.h"

//Prototype Decleration
void vApplicationStackOverflowHook( void );
static void vTaskBlinkLED( void *pvParameters );
static void vTaskADC( void *pvParameters );
static void vTaskLCD( void *pvParameters );

//Queue Decleration

//Handle Decleration
xTaskHandle xHandleLED;
xTaskHandle xHandleADC;
xTaskHandle xHandleLCD;

//Priority Decleration
#define BlinkLED_PRIORITY				( tskIDLE_PRIORITY + 3 )
#define ADC_PRIORITY				( tskIDLE_PRIORITY + 2 )

//Queue Size

//Register Decleration Decleration
unsigned portBASE_TYPE sec = 0;
unsigned int ADCResult, OADCResult, Counter = 0;

//Global Struct Decleration
LCDStat LCDBits;
BOARD_CONTROL BoardControl;

//Global enum Decleration
Mode mode;

//Semaphore Decleration
static xSemaphoreHandle xLCDSem;

const char Digit_Map[10] = {'0', '1', '2', '3', '4', '5', '6', '7', '8', '9'};

_CONFIG2(IESO_OFF & 
 		WUTSEL_FST & 
 		FNOSC_FRCPLL & //FRC with PLL
 		//FNOSC_PRIPLL & //Primary with PLL
 		//FNOSC_PRI & // Primary without PLL
 		FCKSM_CSDCMD & 
 		OSCIOFNC_ON & 
 		IOL1WAY_OFF & 
 		I2C1SEL_PRI &
 		POSCMOD_NONE) //HS OFF
 		//POSCMOD_HS)	//HS ON
 		
_CONFIG1(JTAGEN_OFF & 
		GCP_OFF &
		GWRP_OFF &
		ICS_PGx2 &
		FWDTEN_OFF)
 
void vTaskADC( void *pvParameters )
{		
	unsigned int i = 0;
	portTickType xLastWakeTime;
	
	xLastWakeTime = xTaskGetTickCount(); 
	
	for( ;; )
	{
		vTaskDelayUntil( &xLastWakeTime, 300 );
		
		if( BoardControl.bits.adc_ready == pdTRUE )
		{
			BoardControl.bits.adc_ready == pdFALSE;
			
			ADCResult = 0;
			i = 0;
			while(i <16 )
		   {
				ADCResult = ADCResult + ReadADC10(i);										//Read the conversion results from buffer to ADCResults		
				i++;  
			} 
	
		    ADCResult = ( 484 - ( ( ADCResult / 16 ) - 94 ) ) / 4;
		    ADCResult = ( ADCResult + OADCResult ) / 2;
		    OADCResult = ADCResult;
		    BoardControl.bits.ADC = 1;
		    xSemaphoreGive( xLCDSem );
		}	
	}
}
		
void vTaskBlinkLED( void *pvParameters )
{	
	portTickType xLastWakeTime;
	
	xLastWakeTime = xTaskGetTickCount(); 
	
	for( ;; )
	{		
		vTaskDelayUntil( &xLastWakeTime, 100 );
		vParTestToggleLED( 0 );
	}
}

static void vTaskLCD( void *pvParameters )
{			
	prvSetupLCD();
	
	prvLCDPutString( "www.FreeRTOS.org" );
	vTaskDelay( 50 );
	XLCDCommand(0xC0);
	prvLCDPutString( "    Welcome" );
	
	vTaskDelay( 100 );
	
	prvLCDClear();
	
	XLCDCommand(0x80);
	prvLCDPutString( "Temp = " );
	
	for( ;; )
	{
		if( xSemaphoreTake( xLCDSem, ( portTickType ) 10 ) == pdTRUE )
		{
			if( BoardControl.bits.ADC )
			{
				XLCDCommand( 0x88 );	
				XLCDPut( Digit_Map[ ( ADCResult % 100 ) /100 ] );
				XLCDPut( Digit_Map[ ( ADCResult % 100 ) /10 ] );
				XLCDPut( Digit_Map[ ADCResult % 10 ] );
				XLCDPut( 0b11011111 );
				XLCDPut( 'C' );	
				BoardControl.bits.ADC = 0;
			}
		}
	}
}

void vApplicationStackOverflowHook( void )
{
	/* Look at pxCurrentTCB to see which task overflowed its stack. */
	while (1) 
	{
		portNOP();
	}
}

int main( void )
{	
	prvSetupHardware();
	vPortInitialiseBlocks();

	vSemaphoreCreateBinary( xLCDSem );
	xSemaphoreTake( xLCDSem, ( portTickType ) 10 );
	
	xTaskCreate( vTaskBlinkLED, ( signed char* ) "Blink", configMINIMAL_STACK_SIZE, NULL, BlinkLED_PRIORITY, xHandleLED );
	xTaskCreate( vTaskLCD, ( signed char* ) "LCD", configMINIMAL_STACK_SIZE, NULL, tskIDLE_PRIORITY + 1, xHandleLCD );
	xTaskCreate( vTaskADC, ( signed char* ) "ADC", configMINIMAL_STACK_SIZE, NULL, ADC_PRIORITY, xHandleADC );
		
	prvSetupADC();
	
	vTaskStartScheduler();
	return 0;
}

